#!/usr/bin/perl

# $Id:  $
# Copyright (c) 2013 Network Appliance, Inc.
# All rights reserved.

package Ifconfig;

use strict;
use NetConfig::Interface;
use NetConfig::Route;

=head1 NAME

Ifconfig - Model of the data returned by `ifconfig -a`

=head1 DESCRIPTION

The Ifconfig class models the data returned by `ifconfig -a`;

Use C<Ifconfig-E<gt>parse_ifconfig> to construct a new Ifconfig
object from the current live system.

=head1 SUBROUTINES

=over 4

=item B<new>

    Function   : construct a new Ifconfig object.  Generally you should use parse_interfaces_file instead.
    Returns    : A new empty Ifconfig object.

=cut
sub new {
    my $self = {};
    $self->{INTERFACES} = {};
    bless($self);
    return $self;
}

=item B<add_interface>

    Function   : Adds an Interface to the Ifconfig model
    Parameters : Interface to add

=cut
sub add_interface {
    my $self = shift;
    my ($interface) = @_;

    $self->{INTERFACES}{$interface->name()} = $interface;
}

=item B<get_interface>

    Function   : Gets the Interface with the given name
    Parameters : interface name to retrieve
    Returns    : Interface

=cut
sub get_interface {
    my $self = shift;
    my ($interfaceName) = @_;
    return $self->{INTERFACES}{$interfaceName};
}

=item B<get_interfaces>

    Function   : Gets all the Interfaces
    Returns    : reference to Hash of interfaceName to Interface objects

=cut
sub get_interfaces {
    my $self = shift;
    return $self->{INTERFACES};
}

=item B<parse_ifconfig>

    Function   : Parses the output of `ifconfig -a` into an Ifconfig object
    Returns    : Ifconfig object representation of the output

=cut
sub parse_ifconfig {

    my $ifconfig = new();

    my $inStanza = 0;
    my $currentInterface = undef;
    my $ipv6AddType=undef;

    local *FH;
    open(FH, "ifconfig -a |") or die "Could not run ifconfig: $!\n";
    while (<FH>) {
        chomp;

        if ($_ =~ /^\S.*/) {
            # When we hit a line that does not start with whitespace,
            # we are out of the stanza.
            $inStanza = 0;
        }

        if (!$inStanza) {
            if ($_ =~ /^(\S+):\s+flags=(\S+)\s+.*/) {
                $inStanza = 1;

                my $interfaceName = $1;
                $currentInterface = Interface->new();
                $currentInterface->name($interfaceName);
                $ifconfig->add_interface($currentInterface);
                if (index($_, "MULTICAST") != -1){
                    $ipv6AddType="MULTICAST";
                }
                if (index($_, "UNICAST") != -1){
                    $ipv6AddType="UNICAST"         
                }
            }
        } else {
            # Look for the various options
            # We name these options similar to how they appear in the /etc/network/interfaces file
            if ($_ =~ /\s+inet6\s+([0-9a-f:]+)\s+prefixlen\s+([0-9]+).*<global>/) {
                $currentInterface->set_option("address_ipv6", $1);
                $currentInterface->set_option("prefixlen", $2);
                $currentInterface->set_option("scope","global");
                $currentInterface->set_option("type",$ipv6AddType);

            }

            if ($_ =~ /^\s+inet\s+(\S+)\s+.*/) {
                $currentInterface->set_option("address", $1);
            }
            if ($_ =~ /^\s+inet\s.*\s+netmask\s+(\S+).*/) {
                $currentInterface->set_option("netmask", $1);
            }
            if ($_ =~ /^\s+inet\s+.*\s+broadcast\s+(\S+).*/) {
                $currentInterface->set_option("broadcast", $1);
            }
        }

    }

    close(FH);
    return $ifconfig;
}


=back

=cut

1; # so the require or use succeeds
