#!/bin/bash

#
# If an maintenance user does not exist (e.g. no members of the admins group), then
#     - prompts for a new maintenance username
#     - creates the maintenance user
#     - prompts for the password for the maintenance user
#     - sets the password for the maintenance user
#
# If an maintenance user does exist (e.g. admins group has members), then
#     - do nothing
#

# don't let the user break out
trap "" 1 2 3 4 5 6 9 11 15 20

#
# Verifies that a username is valid.
# The username to check is given as the first argument to this function
#
function verifyUserName() {
    if [[ ${#1} -eq 0 ]] ; then
        echo "You must specify a username."
        return 1
    fi

    if [[ ${#1} -gt 32 ]] ; then
        echo "Username must be 32 characters or less."
        return 1
    fi

    if [[ "$1" =~ .*[[:space:]].* ]] ; then
        echo "Username must not contain whitespace."
        return 1
    fi

    id "$1" > /dev/null 2>&1
    if [[ "$?" -eq 0 ]] ; then
        echo "'$1' is a reserved username.  Select a different username."
        return 1
    fi

    return 0
}

#
# Prompts for a username.
# Returns the provided username in the ${userToAdd} variable.
# Repeats prompting until a valid username is entered.
#
function promptForUserName() {
    while [[ 1 ]]; do
        read -e -r -p "Username: " userToAdd
        verifyUserName "$userToAdd" && return 0
    done
}

#
# Prompts for and adds a user.
# Repeatedly prompts and attempts to add the user until it succeeds.
#
function addUser() {
    while [[ 1 ]]; do
        promptForUserName
        adduser \
            --quiet\
            --shell /bin/maintenance-user-shell.sh\
            --home /home/maintenance\
            --no-create-home\
            --disabled-login\
            --ingroup maintenance\
            --gecos "Maintenance User"\
            "$userToAdd" && return 0
    done
}

#
# Sets the password of the ${userToAdd}
#
function setPassword() {
    while [[ 1 ]]; do
        passwd "$userToAdd" && return 0
    done
}

#
#
# Note that since this is the first authorization unit to be added,
# it will be granted full authority.
#
function setVPCredentials() {
    echo "Setting VP credentials $userToAdd"
}

#
# Add an entry for the maintenance user in the etc/passwd file in the maintenance user's chroot jail.
#
function addMaintenanceUserToJailEtcPasswd() {
    echo "${userToAdd}:x:`id -u ${userToAdd}`:`id -g ${userToAdd}`::/support:/bin/bash" >> /jail/etc/passwd
}


MAINTENANCE_USERS=`members maintenance`
if [[ "x$MAINTENANCE_USERS" != "x" ]] ; then
    # Admin user already created
    # Nothing to do.
    exit 0
fi

cat <<EOF

Create the maintenance user.

The maintenance user manages and maintains the settings
on the virtual appliance.

For example, the maintenance user can do the following:

 - Change network settings
 - Perform upgrades or apply patches

At the prompt, specify the username and password for the new maintenance user.

EOF

addUser
setPassword
setVPCredentials
addMaintenanceUserToJailEtcPasswd
