#!/bin/bash

# $Id:  $
# Copyright (c) 2013 NetApp, Inc.
# All rights reserved.

# Creates a chroot jail for the maintenance user
# to allow the maintenance user to copy support
# bundles to the machine of their choice using
# scp or sftp.
#
# A chroot jail is a minimal filesystem that contains
# enough libraries and stripped down configuration
# files to be able to run a specific set of binaries
# (sftp and scp in our case). Using the chroot command,
# it is possible to lock a user into this file system
# with no way of viewing the root filesystem or accessing
# "real" system files.
# More information can be found here: http://en.wikipedia.org/wiki/Chroot

# The first parameter is the absolute path of the chroot jail.

set -x

JAIL=$1

create_jail_dir() {
    # Create the jail directory if it doesn't exist.

    mkdir -p $JAIL

    if [ -e $JAIL/etc/passwd ]; then
        /usr/bin/chattr -i $JAIL/etc/passwd
    fi

    if [ -e $JAIL/etc/shadow ]; then
        /usr/bin/chattr -i $JAIL/etc/shadow
    fi

    # Delete all files in the jail.
    # Don't remove the jail itself because it may have its own mount point.
    for dir in bin dev etc lib lib64 usr ; do
        rm -rf $JAIL/$dir
    done

    cd $JAIL

    # Copy all required libraries
    mkdir -p $JAIL/lib
    
  cp /lib/x86_64-linux-gnu/libnss_dns.so.2 \
   /lib/x86_64-linux-gnu/libnss_dns-2.24.so \
   /lib/x86_64-linux-gnu/libresolv.so.2 \
   /lib/x86_64-linux-gnu/libresolv-2.24.so \
   /lib/x86_64-linux-gnu/libc.so.6 \
   /lib/x86_64-linux-gnu/libc-2.24.so \
   /lib/x86_64-linux-gnu/ld-linux-x86-64.so.2 \
   /lib/x86_64-linux-gnu/ld-2.24.so \
   /lib/x86_64-linux-gnu/libnsl.so.1 \
   /lib/x86_64-linux-gnu/libnsl-2.24.so \
   /lib/x86_64-linux-gnu/libnss_nisplus.so.2 \
   /lib/x86_64-linux-gnu/libnss_nisplus-2.24.so \
   /lib/x86_64-linux-gnu/libnss_nis.so.2 \
   /lib/x86_64-linux-gnu/libnss_nis-2.24.so \
   /lib/x86_64-linux-gnu/libnss_files.so.2 \
   /lib/x86_64-linux-gnu/libnss_files-2.24.so \
   /lib/x86_64-linux-gnu/libnss_hesiod.so.2 \
   /lib/x86_64-linux-gnu/libnss_hesiod-2.24.so \
   /lib/x86_64-linux-gnu/libnss_compat.so.2 \
   /lib/x86_64-linux-gnu/libnss_compat-2.24.so \
   /lib/x86_64-linux-gnu/libncurses.so.5 \
   /lib/x86_64-linux-gnu/libncurses.so.5.9 \
   /lib/x86_64-linux-gnu/libdl.so.2 \
   /lib/x86_64-linux-gnu/libdl-2.24.so \
   /lib/x86_64-linux-gnu/libtinfo.so.5 \
   /lib/x86_64-linux-gnu/libtinfo.so.5.9 \
   $JAIL/lib > /dev/null 2>&1

    ln -s lib lib64

    # Copy configuration files
    mkdir -p $JAIL/etc
    cp /etc/host.conf \
       /etc/hosts \
       /etc/protocols \
       /etc/services \
       /etc/nsswitch.conf \
       /etc/ld.so.conf \
       $JAIL/etc

    # Create necessary devices
    mkdir -p $JAIL/dev
    #mknod -m 0666 $JAIL/dev/null c `stat -c "%t %T" /dev/null`
    # This doesn't work correctly during install.
    mknod -m 0666 $JAIL/dev/null c 1 3
    mknod -m 0666 $JAIL/dev/urandom c `stat -c "%t %T" /dev/urandom`

    # Copy binaries
    mkdir -p $JAIL/bin
    cp /bin/bash $JAIL/bin

    mkdir -p $JAIL/usr/bin
    cp /usr/bin/scp $JAIL/usr/bin

    mkdir -p $JAIL/usr/lib
    cp /usr/lib/sftp-server $JAIL/usr/lib

    mkdir -p $JAIL/usr/lib/openssh
    cp /usr/lib/openssh/sftp-server $JAIL/usr/lib/openssh

    # Create directory for support bundles
    if [ ! -d $JAIL/support ] ; then
        mkdir -p $JAIL/support
    fi
    chown -R maint:maintenance $JAIL/support

    # create directory for uploads
    if [ ! -d $JAIL/upload ] ; then
        mkdir -m 775 -p $JAIL/upload
    fi
    chgrp maintenance $JAIL/upload

    # Create a symlink to the support bundle directory so that it's easy to find.
    if [ ! -L /support ] ; then
        ln -fs $JAIL/support /support
    fi
}

sync_passwd() {
    # Create a stripped etc/group file with only entries for root and the maintenance group.
    grep "maintenance" /etc/group > $JAIL/etc/group

    if [ -e $JAIL/etc/passwd ]; then
        /usr/bin/chattr -i $JAIL/etc/passwd
    fi

    # Create a stripped etc/passwd file that only contains an entry for root.
    # Note that the addition of the maintenance user to this etc/passwd is deferred to when the maintenance user is created.
    grep "maint" /etc/passwd > $JAIL/etc/passwd
    /usr/bin/chattr +i $JAIL/etc/passwd

    if [ -e $JAIL/etc/shadow ]; then
        /usr/bin/chattr -i $JAIL/etc/shadow
    fi

    grep "maint" /etc/shadow > $JAIL/etc/shadow
    /usr/bin/chattr +i $JAIL/etc/shadow
}

main() {
    if [ "$1" != "-passwd" ] ; then
        create_jail_dir
    fi
    
    sync_passwd
}

main "$@"

