#!/usr/bin/perl

# For OVF parsing
use XML::Parser;

# For option parsing
use Getopt::Long;

# add the current dirname to the lib path so we can find our other modules
use File::Basename;
use lib dirname(__FILE__);
use Common::Text;

$NTP_SERVER_KEY = "vappliance.ntp.servers";
$CONF_FILE = "/etc/ntp.conf";

#
# Print usage and exit
sub doUsage() {
    print "\nusage: firstboot-set-ntp-servers.pl [help|init|check|config]\n";
    print "Set NTP Servers.\n";
    print "\t-help    Print this output and exit.\n";
    print "\t-init    Set NTP servers from OVF data import.\n";
    print "\t-config  Set NTP servers from User input.\n";
    print "\t-check   Nothing.\n";
    exit(99);
}

#
# Parse arguments and return operation mode or fail.
# check - Perform check of setup and prompt user for action if an error is detected.
# config - Prompt user for configuration.
# init - Attempt to read initial config from OVF and prompt user if problem is detected.
#
# @param - none
# @return mode string: 'init', 'check' or 'config'
sub processArguments {
    my $result = "check"; # Default mode

    # Local variables to hold the arg values
    my ($help, $init, $check, $config);

    # Parse command line and get values
    GetOptions(
        'help'   => \$help,
        'init'   => \$init,
        'check'  => \$check,
        'config' => \$config
    );

    # Check in order to give more involved arguments precidence.
    my $num_args = $#ARGV + 1;
    if ($num_args > 0 || $help) {
        doUsage;
    }
    elsif ($config) {
        $result = "config";
    }
    elsif ($init) {
        $result = "init";
    } else {
        return $result;
    }

    return $result;
}

my $ntpserverconfig;

#
# OVF environment XML processing handlers
#
sub OVF_start_handler {
    my $p  = shift;
    my $el = shift;
    while ( my $key = shift ) {
        my $val = shift;
        if ( $key == "oe.key" && $val =~ /vplatform/ ) {
            my $attr = $val;
            $key = shift;
            $val = shift;
            if ( $key == "oe.value" ) {
                $attr =~ s/vplatform\.//;
                $attr =~ s/\.Vendor_Provider_Appliance//;
                $ntpserverconfig{$attr} = $val;
            }
        }
    }
}

#Check other script files where similar function is added as that also needs modification
sub get_vmtoolsd_path() {
	if (-e "/usr/bin/vmtoolsd") {
		return "/usr/bin/vmtoolsd";
	}
	elsif (-e "/usr/sbin/vmtoolsd") {
		return "/usr/sbin/vmtoolsd";
	}
	else {
		return "error";
	}
}

sub OVF_end_handler { }
sub Default_handler { }

sub readNTPServersFromOVF {
    print "\nRetrieving NTP servers from VMware Tools. Please wait...\n";

    # Read the ovf environment
    my $vmtools      = get_vmtoolsd_path();
    my $ovfConfigCmd = $vmtools . " --cmd 'info-get guestinfo.ovfenv' |";
    my $rc           = 0;
    if ( -r $vmtools && -x $vmtools ) {
        open( OVF, $ovfConfigCmd ) or die "Failed to execute vmtools command";
        @ovfenv = <OVF>;
        close(OVF);
        $rc = $?;
        if ( $rc ne 0 ) {
            print "VMware Tools OVF network configuration not found.\n";
            return "error";
        }
    }
    else {
        print "VMware Tools OVF network configuration not found.\n";
        return "error";
    }

    # parse the OVF xml content
    my $parser = new XML::Parser();
    $parser->setHandlers(
        Start   => \&OVF_start_handler,
        End     => \&OVF_end_handler,
        Default => \&Default_handler
    );
    $parser->parse( join( "\n", @ovfenv ) );

    if ( $ntpserverconfig{$NTP_SERVER_KEY} eq "" ) {
        print "No NTP server was given during deployment.\n";
        return "empty";
    }
    
    return "ovf";
}

#
# Set NTPServers
# @returns 0 for OK or non-zero for problem
sub setNTPServersFromOVF {
    my $message    = readNTPServersFromOVF();
    my $retVal         = 0;
    
    if ($message eq "ovf") {
        print("Successfully read from OVF.\n");
        $retVal = doConfig(%ntpserverconfig);
    }
    elsif ($message eq "empty") {
        print "Unable to read NTP servers from OVF.\n";
        $retVal = setVmwareToolsTimeSync(1);
		$retVal = 0;
    
    }
    else {
        print "Error obtaining information from OVF.\n";
        $retVal = setVmwareToolsTimeSync(1);
    }
    
    return $retVal;
}

sub doConfig {
    my %config = @_;
    my @validIp;
    my $rc = 0;
    if ( keys(%config) > 0 ) {
      @validIp = validateServers(%config);
    }
   
    if (@validIp > 0) {
        print "Valid NTP Server IPs: @validIp \n";
        $rc = setNTPServers(@validIp);
    } else {
        setVmwareToolsTimeSync (1);
    }


    return $rc;
}

sub validateServers {
    my %config = @_;
    my $input = $config{$NTP_SERVER_KEY};

    # split the NTP server input field to get the ips separated by comma (hosts).
    my @ipList = split(/,/, $input);
    my @validIp;
    my @invalidIp;
    my $rc = 0;
    
    foreach my $addr (@ipList) {
        $rc = checkAddress($addr);
        if ($rc == 1) {
           push(@validIp, $addr);
        } else {
           print "Invalid NTP server ip : $addr \n";
           push(@invalidIp, $addr);
        }
    }
    
    return @validIp;
}

sub checkAddress() {
    my ($addr) = @_;
    my $retVal = 0;
    
	#First ping the ip to check if it's a valid ip.
	#Then check if the ip is a valid NTP server.
    open(PING, "ping -c 1 $addr |") or die "Failed to execute ping";
    while(<PING>) {
        # Look for the ping success
        if ($_ =~ /1 received/) {
            if (checkIfNTPServer($addr)) {
                $retVal = 1;
            }
        }
    }
    close(PING);
    open(PING6, "ping6 -c 1 $addr |") or die "Failed to execute ping";
    while(<PING6>) {
        # Look for the ping success
        if ($_ =~ /1 received/) {
            if (checkIfNTPServer($addr)) {
                $retVal = 1;
            }
        }
    }
    close(PING6);
    return $retVal;
}

sub checkIfNTPServer {
    my ($addr) = @_;
   
    my $cmd = "ntpq -p $addr";
    my $rc = 0;

    open(NTPQ, "ntpq -p $addr |") or die "Failed to execute ntpq";
    while(<NTPQ>) {
       if ($_ =~ /Connection refused/) {
           $rc = 0;
       } else {
          $rc = 1;
       }
    }
     close(NTPQ);
     
     return $rc;
}

sub setVmwareToolsTimeSync {
    my ($enable) = @_;
    my $cmd;
    my $rc = 0;
    if ($enable) {
       print "Enabling VMwareTools timesync\n";
       $cmd = "/usr/bin/vmware-toolbox-cmd timesync enable";
    } else {
       print "Disabling timesync\n";
       $cmd = "/usr/bin/vmware-toolbox-cmd timesync disable";
    }
   
    $rc = system($cmd) == 0  or die "Couldn't launch [$cmd]: $! / $?";

    return $rc;
   
}

sub setNTPServers {
      my @ipList = @_;
      
      my $addr;
      my $firstAddr;
      my $cmd;
      
      print "Updating NTP Servers list \n";
      
      # For the first NTP ip, change the IP on the following lines of /etc/ntp.conf:
      # server 192.43.244.18 iburst
      # restrict 192.43.244.18 nomodify notrap nopeer noquery
	  # For other ips, add lines below the above statements.
      for my $i (0 .. $#ipList) {
         $addr = $ipList[$i];
         if ($i == 0) {
            $cmd = qq(sed -i "/^server/ cserver $addr iburst
            /^restrict .* nomodify notrap nopeer noquery/ crestrict $addr nomodify notrap nopeer noquery" $CONF_FILE );
            system($cmd) == 0  or die "Couldn't launch [$cmd]: $! / $?";
			
            $firstAddr = $addr
         } else {
            $cmd = qq(sed -i  "/^server $firstAddr iburst/ aserver $addr iburst" $CONF_FILE );
            system($cmd) == 0  or die "Couldn't launch [$cmd]: $! / $?";
			
            $cmd = qq(sed -i  "/^restrict $firstAddr nomodify notrap nopeer noquery/ a restrict $addr nomodify notrap nopeer noquery" $CONF_FILE );
            system($cmd) == 0  or die "Couldn't launch [$cmd]: $! / $?";
         }
      }
	  
}

##################################################################################################################
# Main execution
##################################################################################################################
# At boot, the appliance's startup messages are supressed by redirecting them to an alternate console /dev/ttyS0
# For the interactive portions of this program, we'll need to explicitly switch STDIN and STDOUT to use the real console at /dev/tty1
sub main() {
    open( STDIN,  "</dev/tty1" );
    open( STDOUT, ">/dev/tty1" );

    my $mode = processArguments();
    my $rc   = 0;

    # Clear the screen
    #print (((" " x 80) . "\n") x 25);
    system("clear");

    if ($mode eq "check") {
        # We don't do anything for -check
        exit($rc);
    }

    # Everything else sets NTP Server
    print "\nSet NTP Server\n\n";
    if ( $mode eq "init" ) {
        # Read OVF environment
        $rc = setNTPServersFromOVF();
		print "Restarting ntp server...\n";
		system("systemctl restart ntp");
        if ($rc == 0) {
            exit(0);
        } else {
            print "\n";
            print "No NTP Server was retrieved during OVF prompts.\n";
            print "This could be due to either an issue with VMware Tools or\n";
            print "network configuration issue.\n";
            print "Settiung VMwareTools timesync.\n ";
            print "\n";
            
            setVmwareToolsTimeSync(1);
        }
    }    #End init
    elsif( $mode eq "config" ) {
        doConfig();
    }  # End config
        
    exit(0);
}

main;

